/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Virtual server
*  SCOPE :              Virtual server
*  FILE :               virtualdbserver.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __virtualdbserver_HXX
#define __virtualdbserver_HXX

/* IMPORTS *******************************************************************/

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __pagehdl_HXX
#include "pagehdl.hxx"
#endif

#ifndef __dbvue_hxx
#include "dbvue.hxx"
#endif

#ifndef __config_H
#include "config.h"
#endif

/*****************************************************************************/

class VirtualDBServer {

  private :

  int _delta ;
  char* _name ;
  DBVue* _tabvue[MAX_VOL] ;
  char*  _tabname[MAX_VOL] ;
  ListPtr<PageHdlMem> _touchPage ;
  PageSet _keep ;
  ListVal<DBVue> _vues ;
  DBVue* server(int vol) ;
  
  public :

  VirtualDBServer(const char* name) ;
  ~VirtualDBServer() ;
  void createVue(const char* server, ListPtr<Volume>*) ;
  void eject(PID&) ;
  void touchPage(PageHdlMem*) ;           // Insert page in touch page list
  int lock(PID&, Lock) ;                  // Attempt to lock a page
  void readfirst(Page*, PID&) ;           // Attempt to load a page
  void readnext(Page*, PID&) ;            // Attempt to load a page
  PageHdlMem* allocation(Cluster*, int) ; // Allocation of pages
  Page* findPage(PID) ;                   // Load page in virtual memory
  int checkpoint() ;                      // Checkpoint current transaction
  int upgrade(PageHdlMem**) ;             // Upgrade current transaction
  int abort() ;
  int release(PageHdlMem**) ;
  int secondphase() ;
  int sync() ;
  void setdelta(int d) { _delta = d ; }
} ;


inline void VirtualDBServer::touchPage(PageHdlMem* pd) {
  this->_touchPage += pd ;
}

inline void VirtualDBServer::eject(PID& pid) {
  this->_keep -= pid ;
}

#endif
